@@ -8,6 +8,7 @@ from loguru import logger
 def add_common_args(parser):
 
     # --------------- Command arguments ---------------
+    parser.add_argument("--do_new", action="store_true", help="New project")
     parser.add_argument("--do_train",
                         action="store_true",
                         help="Whether to run training.")
@@ -57,6 +58,8 @@ def add_common_args(parser):
         help=
         "The input data dir. Should contain the training files for the CoNLL-2003 NER task.",
     )
+
+    parser.add_argument("--brat_data_dir", default=None)
     parser.add_argument(
         "--output_dir",
         type=str,
@@ -71,6 +74,25 @@ def add_common_args(parser):
         #  required=True,
         help="The submissions dir.",
     )
+    parser.add_argument(
+        "--experiments_dir",
+        type=str,
+        default="./experiments",
+        #  required=True,
+        help="The experiments dir.",
+    )
+    parser.add_argument(
+        "--experiment_id",
+        type=str,
+        default=None,
+        help="The experiment ID.",
+    )
+    parser.add_argument(
+        "--model_id",
+        type=str,
+        default=None,
+        help="The model ID.",
+    )
     parser.add_argument(
         "--cache_dir",
         type=str,
@@ -232,6 +254,16 @@ def add_common_args(parser):
                         type=float,
                         help="train sample rate.")
 
+    parser.add_argument("--max_train_examples",
+                        default=0,
+                        type=int,
+                        help="Max number of train examples.")
+    parser.add_argument("--confidence",
+                        default=0.5,
+                        type=float,
+                        help="Probs confidence.")
+    parser.add_argument("--enable_nested_entities", action="store_true")
+
     # ------------------------------
     # Knowledge distillation
     parser.add_argument("--enable_kd",
@@ -252,6 +284,14 @@ def add_common_args(parser):
                         type=int,
                         default=2,
                         help="KD SDA teachers.")
+    parser.add_argument(
+        "--sda_stategy",
+        type=str,
+        default="recent_models",
+        help=
+        "KD SDA stategy [recent_models|earliest_models|latest_model|clone_models]."
+    )
+    parser.add_argument("--sda_empty_first", action="store_true")
     parser.add_argument("--sda_coeff",
                         type=float,
                         default=1.0,
@@ -261,6 +301,9 @@ def add_common_args(parser):
                         default=0.995,
                         help="The exponential decay of KD.")
 
+    parser.add_argument("--generate_submission", action="store_true")
+    parser.add_argument("--dataset_file", type=str, default=None)
+    parser.add_argument("--submission_file", type=str, default=None)
     # ------------------------------
     parser.add_argument("--server_ip",
                         type=str,
@@ -330,19 +373,42 @@ def add_common_args(parser):
     parser.add_argument("--seg_len", type=int, default=256, help="")
     parser.add_argument("--seg_backoff", type=int, default=64, help="")
     parser.add_argument("--max_span_len", type=int, default=32, help="")
-    parser.add_argument("--num_augements", type=int, default=0, help="")
+    parser.add_argument("--num_augments", type=int, default=0, help="")
+    parser.add_argument("--aug_train_only", default=False, help="")
     parser.add_argument(
         "--loss_type",
         type=str,
         default="CrossEntropyLoss",
         help=
-        "Loss type: ['CrossEntropyLoss', 'FocalLoss', 'LabelSmoothingCrossEntropy', 'CircleLoss']"
+        "Loss type: ['CrossEntropyLoss', 'FocalLoss', 'DiceLoss', 'LabelSmoothingCrossEntropy', 'CircleLoss']"
     )
+    parser.add_argument("--diceloss_weight", default=None)
     parser.add_argument("--focalloss_gamma", type=float, default=2.0)
     parser.add_argument("--focalloss_alpha", type=float, default=None)
     parser.add_argument("--allow_overlap", action="store_true", help="")
-    parser.add_argument("--random_type", type=str, default=None, help="[None, 'np']")
+    parser.add_argument("--random_type",
+                        type=str,
+                        default=None,
+                        help="[None, 'np']")
+    parser.add_argument("--is_english", action='store_true')
 
+    parser.add_argument("--to_train_poplar", action="store_true")
+    parser.add_argument("--to_reviews_poplar", action="store_true")
+    parser.add_argument("--start_page", type=int, default=0)
+    parser.add_argument("--max_pages", type=int, default=100)
+
+    parser.add_argument("--reviews_file", type=str, default=None)
+
+    parser.add_argument("--emotion_words_file",
+                        type=str,
+                        default=None,
+                        help="Emotion words file.")
+
+    parser.add_argument("--cc",
+                        default=None,
+                        type=str,
+                        choices=['t2s', 's2t', 'mix2t', 'mix2s'],
+                        help="OpenCC")
     return parser
 
 
@@ -384,6 +450,10 @@ def get_main_args(
         if not os.path.exists(args.submissions_dir):
             os.makedirs(args.submissions_dir)
 
+    if args.experiments_dir:
+        if not os.path.exists(args.experiments_dir):
+            os.makedirs(args.experiments_dir)
+
     #  if not os.path.exists(args.local_dir):
     #      os.makedirs(args.local_dir)
 
@@ -400,7 +470,7 @@ def get_main_args(
             os.makedirs(args.latest_dir)
         if not os.path.exists(args.local_id_file):
             import uuid
-            local_id = str(uuid.uuid1()).replace('-', '')
+            local_id = str(uuid.uuid1()).replace('-', '')[:8]
             args.local_id = local_id
             with open(args.local_id_file, 'w') as wt:
                 wt.write(f"{local_id}")
@@ -416,16 +486,31 @@ def get_main_args(
         #  os.symlink(local_id, args.latest_dir)
 
     ensure_latest_dir(args)
-    args.local_dir = os.path.join(args.output_dir, args.local_id)
+    if args.model_id is None:
+        args.model_id = args.local_id
+
+    args.saved_models_path = os.path.join(args.output_dir, "saved_models")
+    if not os.path.exists(args.saved_models_path):
+        os.makedirs(args.saved_models_path)
     args.best_model_path = os.path.join(args.latest_dir, "best")
+    args.local_dir = os.path.join(args.saved_models_path, args.local_id)
 
     logname = args.task_name
     logger.add(os.path.join(args.latest_dir, f"{logname}.log"))
 
+    if args.experiment_name is None:
+        args.experiment_name = "Theta"
+    if args.dataset_name is None:
+        args.dataset_name = os.path.basename(os.path.abspath(os.path.curdir))
+
+    args.reviews_file = f"{args.latest_dir}/{args.dataset_name}_reviews_{args.local_id}.json"
+
     logger.warning(f"dataset_name: {args.dataset_name}")
     logger.warning(f"experiment_name: {args.experiment_name}")
     logger.warning(f"local_id: {args.local_id}")
     logger.warning(f"local_dir: {args.local_dir}")
     logger.warning(f"latest_dir: {args.latest_dir}")
+    logger.warning(f"experiments_dir: {args.experiments_dir}")
+    logger.warning(f"saved_models_path: {args.saved_models_path}")
 
     return args
